/*
 * Copyright (C) 2005 Andre Noll <maan@systemlinux.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#include "gcc-compat.h"
#include "para.h"

#include "http.h"
#include "recv.h"

#define BUFSIZE 8192
enum {HTTP_DISCONNECTED, HTTP_CONNECTED, HTTP_SENT_GET_REQUEST, HTTP_STREAMING};

/* no encryption */
void (*crypt_function_recv)(unsigned long len, const unsigned char *indata, unsigned char *outdata) = NULL;
void (*crypt_function_send)(unsigned long len, const unsigned char *indata, unsigned char *outdata) = NULL;

struct private_http_data {
	int status;
	int fd;
};

static void http_shutdown(void)
{
	return;
}

static char *make_request_msg(void)
{
	char *ret, *hn = para_hostname();
	ret = make_message("%s\nHost: %s\nUser-Agent: para_recv/%s\n\n\n",
		HTTP_GET_MSG, hn, VERSION);
	free(hn);
	return ret;
}

static int http_pre_select(struct receiver_node *rn, fd_set *rfds, fd_set *wfds,
	__unused struct timeval *timeout)
{
	struct private_http_data *phd = rn->private_data;

	if  (phd->status == HTTP_CONNECTED) {
		PARA_NOTICE_LOG("setting fd %d for write\n", phd->fd);
		FD_SET(phd->fd, wfds);
	} else
		FD_SET(phd->fd, rfds);
	return phd->fd;
}

static int http_post_select(struct receiver_node *rn, int select_ret,
		fd_set *rfds, fd_set *wfds)
{
	int ret;
	struct private_http_data *phd = rn->private_data;

	if (!select_ret) /* we're not interested in timeouts */
		return 1;
	if  (phd->status == HTTP_CONNECTED) {
		char *rq;
		if (!FD_ISSET(phd->fd, wfds))
			return 1; /* nothing to do */
		rq = make_request_msg();
		PARA_NOTICE_LOG("sending request message:\n%s\n", rq);
		ret = send_va_buffer(phd->fd, "%s", rq);
		free(rq);
		if (ret < 0)
			return E_WRITE;
		phd->status = HTTP_SENT_GET_REQUEST;
		return 1;
	}
	if (!FD_ISSET(phd->fd, rfds))
		return 1; /* nothing to do */
	if (phd->status == HTTP_SENT_GET_REQUEST) {
		ret = recv_pattern(phd->fd, HTTP_OK_MSG, MAXLINE);
		if (ret < 0)
			return -E_MISSING_OK;
		PARA_NOTICE_LOG("%s", "received ok msg, streaming\n");
		phd->status = HTTP_STREAMING;
		return 1;
	}
	/* already streaming */
	if (rn->loaded >= BUFSIZE)
		return -E_OVERRUN;
	ret = recv_buffer(phd->fd, rn->buf, BUFSIZE - rn->loaded);
	if (ret <= 0)
		return ret < 0? -E_READ : 0;
	rn->loaded += ret;
	return 1;
}

static void http_recv_close(struct receiver_node *rn)
{
	struct private_http_data *phd = rn->private_data;
	close(phd->fd);
	free(rn->buf);
	free(rn->private_data);
}

static int http_recv_open(struct receiver_node *rn)
{
	struct private_http_data *phd;
	struct hostent *he;
	struct sockaddr_in their_addr;
	int ret;

	rn->buf = para_malloc(BUFSIZE);
	rn->loaded = 0;
	rn->private_data = para_calloc(sizeof(struct private_http_data));
	phd = rn->private_data;

	/* get the host info */
	PARA_NOTICE_LOG("getting host info of %s\n", rn->host);
	ret = -E_HOST_INFO;
	if (!(he = get_host_info(rn->host)))
		goto err_out;
	/* get new socket */
	ret = -E_SOCKET;
	if ((phd->fd = get_socket()) < 0)
		goto err_out;
	/* init their_addr */
	init_sockaddr(&their_addr, rn->port, he);
	/* connect */
	PARA_NOTICE_LOG("connecting to %s:%d\n", rn->host, rn->port);
	ret = -E_CONNECT;
	if (do_connect(phd->fd, &their_addr) < 0) {
		PARA_ERROR_LOG("connect failed (%s)\n", strerror(errno));
		goto err_out;
	}
	phd->status = HTTP_CONNECTED;
	return 1;
err_out:
	free(rn->private_data);
	free(rn->buf);
	return ret;
}

int http_recv_init(struct receiver *r)
{

	r->open = http_recv_open;
	r->close = http_recv_close;
	r->pre_select = http_pre_select;
	r->post_select = http_post_select;
	r->shutdown = http_shutdown;
	return 1;
}
